//============================================================================
// Edward Thompson
// 10/16/2005
//
// Homework Assignment 4
//          CR320
//
// StockQuoteViewer
//      GUI for the display of stock quote information.  Will save the users
//  list of stock tickers upon exit, and load them up upon start.
//============================================================================
package homework4;

import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import gui.run.RunJob;
import gui.layouts.PreferredSizeGridLayout;
import net.stocks.Quote;

import javax.swing.*;
import java.awt.*;
import java.util.Vector;

import utils.PreferencesUtils;

public class StockQuoteViewer extends gui.ClosableJFrame
{
    private Vector quotes;//local vector for gui display
    private Vector tickernames;// = new Vector();//vector to hold just the ticker strings

    //gui instance object references - tracked for usage by all instance methods
    private StockQuoteViewer sqv = null;
    private JPanel labelPanel;
    private JLabel[] labels;

    private String prefs_key = "stock_quote_viewr_key";

    public StockQuoteViewer( String title )
    {
        super(title);
        tickernames = getTickerList();
    }

    //*************************************************************************
    //GUI initialization methods
    public void init()
    {
        setDisplayVector(getQuotes(tickernames));

        setContainerLayout( new BorderLayout() );
        Container c = getContentPane();
        c.add( setJMenuBar(), BorderLayout.NORTH );
        setBackground( Color.white );
        setDisplayLabels();
        pack();
        setVisible( true );

        sqv = this;
    }

    private RunMenuBar setJMenuBar()
    {
        RunMenuBar mb = new RunMenuBar();
        mb.add( getFileMenu() );
        mb.add( getEditMenu() );
        return mb;
    }

    private RunMenu getFileMenu()
    {
        RunMenu fileMenu = new RunMenu( "[File" );
        fileMenu.add( new RunMenuItem( "[quit{control q}" )
        {
            public void run()
            {
                //this will save the current list of stocks, then exit
                saveTickerList();
                System.exit( 0 );
            }
        } );
        return fileMenu;
    }

    private RunMenu getEditMenu()
    {
        RunMenu editMenu = new RunMenu( "[Edit" );
        editMenu.add( new RunMenuItem( "[customize{control c}")
        {
            public void run()
            {
                //should open up a subwindow that allows the user to enter a list of stocks
                tickernames = StockQuoteUtils.createStockListWindow( tickernames, sqv );
                setDisplayVector(getQuotes(tickernames));

                //need a method here to repaint the GUI properly, resize the menu, etc
                changeDisplayLabels();
            }
        } );
        return editMenu;
    }

    //creates a new display panel displaying information for all
    // instance display vector elements
    private void setDisplayLabels()
    {
        labelPanel = new JPanel();

        labels = new JLabel[getDisplayVector().size()];
        for ( int i = 0; i < getDisplayVector().size(); i++ )
        {
            String s = (String) getDisplayVector().elementAt( i );
            labels[i] = new JLabel( s );
            labelPanel.add( labels[i] );
        }
        labelPanel.setLayout( new PreferredSizeGridLayout(0, 1, 5, 10) );

        addComponent( labelPanel, BorderLayout.SOUTH );
    }
    //END GUI initialization methods
    //*************************************************************************

    //*************************************************************************
    //GUI update methods
    //
    // updates the display panel - removes everything in it previously,
    // and then creates a new display panel with the display vectors
    // for all instance display vector elements
    private void changeDisplayLabels()
    {
        labelPanel.removeAll();

        labels = new JLabel[getDisplayVector().size()];
        for ( int i = 0; i < getDisplayVector().size(); i++ )
        {
            String s = (String) getDisplayVector().elementAt( i );
            labels[i] = new JLabel( s );
            labelPanel.add( labels[i] );
        }
        labelPanel.setLayout( new PreferredSizeGridLayout(0, 1, 5, 10) );

        addComponent(labelPanel, BorderLayout.SOUTH);
        pack();
    }

    //Updates the display by going to the web and getting the new quotes based on
    // the instance vector tickernames
    public void updateDisplay()
    {
        setDisplayVector(getQuotes(tickernames));
        for ( int i = 0; i < getDisplayVector().size(); i++ )
        {
            labels[i].setText( (String) getDisplayVector().elementAt( i ) );
        }
    }

    public void paint( Graphics g )
    {
        new RunJob( 2, false )
        {
            public void run()
            {
                updateDisplay();
            }
        };
        super.paint( g );
    }
    //END GUI update methods
    //*************************************************************************

    //saveTickerList will save the vector tickernames into the users preferences
    private void saveTickerList()
    {
        PreferencesUtils prefs =  new PreferencesUtils( prefs_key );

        prefs.save( tickernames );
    }

    //getTickerList will retrieve the vector of ticker name strings from the
    // users preferences
    private Vector getTickerList()
    {
        Vector tickerList;

        PreferencesUtils prefs =  new PreferencesUtils( prefs_key );

        tickerList = (Vector) prefs.restore();

        if( tickerList == null )
        {
            tickerList = new Vector();
        }

        return tickerList;
    }

    //setDisplayVector and getDisplayVector are synchronized methods because
    // the paint method utilizes a separate thread
    private synchronized void setDisplayVector( Vector v )
    {
        quotes = v;
    }

    private synchronized Vector getDisplayVector()
    {
        return quotes;
    }

    public static Vector getQuotes( Vector tickers )
    {
        return Quote.getUrl( Quote.makeQuoteURLString( tickers ) );
    }

    public static void main( String args[] )
    {
        try
        {
            //set the look and feel to the system look and feel for prettiness
            UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception e)
        {
            System.out.println(e);
        }

        StockQuoteViewer qf = new StockQuoteViewer( "Stock Quote Viewer" );
        qf.init();
    }
}